      program tstemp ! bernd berg, jan 21 2020.
c Reference: Bernd A. Berg, ``Warming or cooling from a random 
c                             walk process in the temperature"
c Generates nrpt temperature random walks: 
c 1. Small Gaussian fluctuations with standard deviation sigma1=0.02. 
c 2. Adding large modified Gaussian fluctuations (driven back to zero)
c    with standard deviation sigma2=0.1.
      include 'implicit.sta'
      include 'constants.par'
c cpick:   tstemp file name extension ipick.
      character cpick*4
c nrpt:      number of random walks created. Labelled irpt=1,...,nrpt.
c ipick:     irpt value for which gnuplot txt files are prepared. See
c            readme.txt for explanation of the different ipick values.
c n:         length of temperature random walk (units of month).
      parameter(nrpt=10001,n=2040) ! Parameters Used in paper.
      parameter(ipick=0718) ! resembles observed temperature. Used.
c     parameter(ipick=7691) ! extreme temperature increase.   Used.
c     parameter(ipick=1902) ! smallest temperature |T2-T1|. Not Used.
c     parameter(ipick=8293) ! smallest temperature Tmax-Tmin. Used.
c     parameter(ipick=6515) ! extreme temperature decrease.   Used.
c n1,n2:     symmetric 95% confidence range.
      parameter(n1=1+0.025*n,n2=1+n-n1)
c iuo:     output unit.
c iud*:    data units.
c isd*:    seeds for Marsaglia random number generation.
      parameter(iuo=6,iud1=11,iud2=12,iud3=13,isd1=1,isd2=1)
c     parameter(iuo=6,iud1=11,iud2=12,iud3=13,isd1=2,isd2=2)
c sigma1, sigma2: standard deviations of small and large fluctuations.
      parameter(sigma1=0.02d00,sigma2=0.1d00)
c t0:        shift of entire curve by t0.
c tfactor:   = a in paper. Parameter for driving fluctuations back.
c to2to1:    observed temperature range To2-To1 Eq.(10) of paper.
      parameter(t0=-0.5d0,tfactor=4.d0,to2to1=0.99d00)
c dtgau(), tgau():    small Gaussian updates and temperatures.
c dtmod(), tmod(): modified Gaussian updates and temperatures.
c t():             added temperatures.
      dimension dtgau(n),tgau(n),dtmod(n),tmod(n),t(n)
c t2mt1m:     average temperatures from first and last 2.5% range.
      dimension t2mt1m(nrpt),iper(nrpt),jper(nrpt)
c tmax(), tmin(): Minimum and maximum temperatures for each random walk.
c tmatmi():       tmax()-tmin() for each random walk, sorted on file.
      dimension tmax(nrpt),tmin(nrpt),tmatmi(nrpt)
c nhot:   number of random walks for which the warming equals 
c         or exceeds the observed warming.
      data tgau(1)/zero/, tmod(1)/zero/, nhot/0/
      data tmax/nrpt*zero/, tmin/nrpt*zero/, tmatmi/nrpt*zero/
      write(cpick,'(i4.4)') ipick
      ltest=.true.
      ltest=.false.
      if(ltest) stop "tstemp: ltest."
c
      write(iuo,'(/,"program tstemp.f: ipick,nrpt,n,tfactor =",
     &                   3i6,f8.3,/)') ipick,nrpt,n,tfactor
      call rmaset(iuo,iud1,isd1,isd2,"no file")
      write(iuo,'(/,"gnuplot tsgautemp.plt,tsmodtemp.plt,",
     &  "tstempcpick.plt (cpick number ipick),")')
      write(iuo,'(  "  files tsgautemp.tst,tsmodtemp.txt,",
     &  "tstempcpick.txt (cpick number ipick).")')
      open(iud1,file="tsgautemp.txt")
      open(iud2,file="tsmodtemp.txt")
      open(iud3,file="tstemp"//cpick//".txt")
      do irpt=1,nrpt
        nh=n/2
        do i=1,nh
          call rmagau(dt1,dt2)
          i1=2*i-1
          i2=2*i
          dtgau(i1)=sigma1*dt1
          dtgau(i2)=sigma1*dt2
        enddo
        do i=2,n
          im1=i-1
          tgau(i)=tgau(im1)+dtgau(im1)
        enddo
c
        do i=1,n ! Loop over random walk length=n.
          call rmagau(dx,xgr)
          absdt=abs(sigma2*dx)
          if(i.ge.2) then 
            im1=i-1
            if(xgr.lt.tfactor*tmod(im1)) then
              dtmod(im1)=-absdt
            else
             dtmod(im1)=+absdt
            endif
            tmod(i)=tmod(im1)+dtmod(im1) ! tmod(1) is never changed.
          endif
          t(i)=tmod(i)+tgau(i)
          tmax(irpt)=max(t(i),tmax(irpt))
          tmin(irpt)=min(t(i),tmin(irpt))
          t(i)=t(i)+t0
c Fluctuations of the entire random walk are kept on file for irpt=ipick:
          if(irpt.eq.ipick) then 
            tdate=1850.00+i/12.0d00
            write(iud1,'(i4,f12.4,2f12.5)') i,tdate,tgau(i),dtgau(i)
            write(iud2,'(i4,f12.4,2f12.5)') i,tdate,tmod(i),dtmod(i)
            write(iud3,'(i4,f12.4,1f12.5)') i,tdate,t(i)
          endif
        enddo ! End of loop over random walk length.
c
        tmatmi(irpt)=tmax(irpt)-tmin(irpt)  ! Max-Min T for rw irpt.
        call steb0(n1,t(01),t1m,t1mv,t1me)    ! t1m=T1 of paper.
        call steb0(n1,t(n2),t2m,t2mv,t2me)    ! t2m=T2 of paper.
        t2mt1m(irpt)=t2m-t1m
        if(t2mt1m(irpt).gt.to2to1) nhot=nhot+1
      enddo
      close(iud1)
      close(iud2)
      close(iud3)
c
      write(iuo,'(/,"n1,n2,t1m,t2m,t2m-t1m:",2i6,3f9.4)')
     &               n1,n2,t1m,t2m,t2m-t1m
      write(iuo,'(/,"nhot =",14x,i8,/)') nhot
c
      call heap_per(nrpt,tmatmi,iper)
      write(iuo,'("write sorted tmatmi on file tmatmiper.txt")')
      write(iuo,'("-> ipick = iper(1) =",i6,/)') iper(1)
      call heap_per(nrpt,t2mt1m,jper)
      write(iuo,'("write sorted t2mt1m on file t2mt1mper.txt")')
      write(iuo,'("-> ipick = jper(1),jper(5047),jper(nrpt):",3i6,/)')
     &                        jper(1),jper(5047),jper(nrpt)
      open(iud1,file="tmatmiper.txt")
      open(iud2,file="t2mt1mper.txt")
      do irpt=1,nrpt
        write(iud1,'(2i9,1f12.5)') irpt,iper(irpt),tmatmi(irpt)
        write(iud2,'(2i9,1f12.5)') irpt,jper(irpt),t2mt1m(irpt)
      enddo
      close(iud1)
      close(iud2)
c
      nhot=nrpt+1-nhot
      write(iuo,'("Hot temperature t2mt1m >=  ",f9.4)') t2mt1m(nhot)
      med=nrpt/2+1
      write(iuo,'("Medium temperature t2mt1m =",f9.4,/)') t2mt1m(med)
c
      call df_gnu(iud1,nrpt,t2mt1m)
c
      stop "tstemp program: all done."
      end

      include 'forlib/df_gnu.f'
      include 'forlib/heap_per.f'
      include 'forlib/ranmar.f'
      include 'forlib/rmagau.f'
      include 'forlib/rmaset.f'
      include 'forlib/steb0.f'
